import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { pathOr, when, is, isEmpty, pipe, keys, filter, map, __, prop } from 'ramda';
import Dropdown from 'reactstrap/lib/Dropdown';
import DropdownToggle from 'reactstrap/lib/DropdownToggle';
import DropdownMenu from 'reactstrap/lib/DropdownMenu';
import DropdownItem from 'reactstrap/lib/DropdownItem';
import classNames from 'classnames';

import { LANG_NAMES, LANG_KEYS } from '../../constants';
import { IconArrowDown } from '../icons/icon-arrow-down/icon-arrow-down';
import { IconSelected } from '../icons/icon-selected/icon-selected';
import { changeLocation, getLocationOrigin } from '../../helpers/location';

import './lang-switcher.scss';

/**
 * Generates locales list
 *
 * @param {{}|[]} localesList
 * @param {boolean} isMultiDomains
 * @param {string} defaultDomain
 * @returns {{}}
 */
export const generateLocalesList = (localesList, isMultiDomains, defaultDomain = 'com') => {
  const targetLocales = {};
  const localesKeys = Array.isArray(localesList) ? localesList : Object.keys(localesList);

  localesKeys.forEach((locale) => {
    targetLocales[locale] = {
      title: LANG_NAMES[locale],
      shortCode: LANG_KEYS[locale],
      domain: isMultiDomains ? (localesList[locale] || defaultDomain) : null,
    };
  });

  return targetLocales;
};

/**
 * Parses intl key with locales (special structure) and generates list
 *
 * @param {string} localesStrKey
 * @param errorHandler - custom callback
 * @returns {{}|null}
 */
export const parseLocalesIntlKey = (localesStrKey, errorHandler = null) => {
  const regex = /^([a-z-]+)\|([^|]+)\|([a-z]{2,})\|([a-z.]{2,})(\|([^|\n\r]*))?$/mg;
  const targetLocales = {};
  let regexMatch;

  while ((regexMatch = regex.exec(localesStrKey)) !== null) { // eslint-disable-line no-cond-assign
    if (regexMatch.index === regex.lastIndex) {
      regex.lastIndex += 1;
    }

    const {
      1: locale,
      2: title,
      3: shortCode,
      4: domain,
    } = regexMatch;

    targetLocales[locale] = {
      title,
      shortCode,
      domain,
    };
  }

  if (isEmpty(targetLocales)) {
    when(is(Function), f => f())(errorHandler);
    return null;
  }

  return targetLocales;
};

export class LangSwitcher extends Component {
  static propTypes = {
    locale: PropTypes.string.isRequired,
    defaultLocales: PropTypes.oneOfType([PropTypes.objectOf(PropTypes.string), PropTypes.arrayOf(PropTypes.string)]),
    domain: PropTypes.string,
    localesIntl: PropTypes.string.isRequired,
    localesIntlErrorHandler: PropTypes.func,
    isWithinHeader: PropTypes.bool,
    isWhite: PropTypes.bool,
    className: PropTypes.string,
    buttonClassName: PropTypes.string,
  };

  static defaultProps = {
    defaultLocales: Object.keys(LANG_NAMES),
    isWithinHeader: false,
    isWhite: false,
    localesIntlErrorHandler: null,
    domain: 'com',
    className: null,
    buttonClassName: null,
  };

  state = {
    dropdownOpen: false,
  };

  isMultiDomains = !Array.isArray(this.props.defaultLocales);

  toggle = () => {
    this.setState(prevState => ({
      dropdownOpen: !prevState.dropdownOpen,
    }));
  };

  changeLocale = (e) => {
    const { locale, domain } = this.props;
    const newLocale = e.target.value;

    if (newLocale !== locale) {
      const url = this.isMultiDomains ?
        getLocationOrigin().replace(new RegExp(`(.*)${domain.toLowerCase()}`), `$1${pathOr('com', [newLocale, 'domain'], this.localesList)}`) :
        `${getLocationOrigin()}${window.location.pathname.replace(locale, newLocale)}`;

      changeLocation(url);
    }
  };

  localesList = {};

  render() {
    const { dropdownOpen } = this.state;
    const {
      locale,
      isWithinHeader,
      isWhite,
      defaultLocales,
      localesIntl,
      localesIntlErrorHandler,
      className,
      buttonClassName,
    } = this.props;
    const parsedIntlKeyList = parseLocalesIntlKey(localesIntl, localesIntlErrorHandler);
    this.localesList = parsedIntlKeyList || generateLocalesList(defaultLocales, this.isMultiDomains);

    return (
      <Dropdown
        isOpen={dropdownOpen}
        toggle={this.toggle}
        className={classNames(className, 'lang-switcher', { 'lang-switcher-within-header': isWithinHeader, 'lang-switcher-white': isWhite })}
      >
        <DropdownToggle
          nav
          className={classNames(buttonClassName, 'lang-switcher-button p-0')}
        >
          {!isWithinHeader ? `${this.localesList[locale].title}` : this.localesList[locale].shortCode.toUpperCase()}
          <IconArrowDown className={classNames('lang-switcher-arrow', { 'align-middle': isWithinHeader, 'position-relative': !isWithinHeader })} />
        </DropdownToggle>

        <DropdownMenu className="language-dropdown">
          {
            pipe(
              keys,
              filter(prop(__, LANG_NAMES)),
              map((loc) => {
                const isActive = loc === locale;

                return (
                  <DropdownItem
                    key={loc}
                    value={loc}
                    active={isActive}
                    className={classNames('px-2', {
                      'font-weight-bold d-flex justify-content-between align-items-center category-selected': isActive,
                    })}
                    onClick={this.changeLocale}
                  >
                    {this.localesList[loc].title}
                    {isActive && <IconSelected />}
                  </DropdownItem>
                );
              }),
            )(this.localesList)
          }
        </DropdownMenu>
      </Dropdown>
    );
  }
}
