import React from 'react';
import { LangSwitcher, parseLocalesIntlKey } from './lang-switcher';
import { changeLocation, getLocationOrigin } from '../../helpers/location';

jest.mock('../../helpers/location');

const localesIntl = `en-us|English 1|en|com|Test desc
ru-ru|Русский|ru|me
me-me||me|me
zh-cn|中文|zh|net
es-es|Español|es|es
ko-kr|한국어|ko|kr
en-in|Indian|en|in
tr-tr|Türkçe|tr|tc
fr-fr|Français|fr|fr
it-it|Italiano|it|it
uk-ua|Українська|uk|com.ua
pl-pl|Polski|pl|pl
ba-ba|Balkanski|ba|ba
en-ph|Filipino|en|ph
de-de|Deutsch|de|de
nl-nl|Nederlands|nl|com.nl
id-id|Bahasa Indonesia|id|id
vi-vn|Tiếng Việt|vi|vn
ro-ro|Română|ro|ro
en-africa|Africa|en|africa`;

describe('<LangSwitcher />', () => {
  const wrapper = shallow(<LangSwitcher localesIntl={localesIntl} locale="en-us" domain="com" />);

  beforeEach(() => {
    changeLocation.mockClear();
  });

  test('should toggle dropdown', () => {
    wrapper.instance().toggle();
    expect(wrapper.instance().state.dropdownOpen).toBe(true);
  });

  test('should not change location when locale equals new locale', () => {
    const event = {
      target: {
        value: 'en-us',
      },
    };
    wrapper.instance().changeLocale(event);
    expect(changeLocation).not.toHaveBeenCalled();
  });

  test('should change location when locale not equal new locale', () => {
    const event = {
      target: {
        value: 'ru-ru',
      },
    };
    wrapper.instance().changeLocale(event);

    expect(changeLocation).toHaveBeenCalled();
  });

  describe('common snapshots', () => {
    describe('when `localesIntl` prop is valid string with spec structure', () => {
      describe('and `defaultLocales` prop is Object', () => {
        test('should match snapshot', () => {
          const wrapper2 = shallow(<LangSwitcher
            localesIntl={localesIntl}
            defaultLocales={{
              'en-us': 'com',
              'ru-ru': 'ru',
              'es-es': 'es',
              'zh-cn': 'net',
              'ro-ro': 'ro',
              'ko-kr': 'kr',
              'pl-pl': 'pl',
              'fr-fr': 'fr',
            }}
            locale="en-us"
            domain="com"
          />);

          expect(wrapper2).toMatchSnapshot();
        });
      });

      describe('and `defaultLocales` prop is Array', () => {
        test('should match snapshot', () => {
          const wrapper2 = shallow(<LangSwitcher
            localesIntl={localesIntl}
            defaultLocales={['en-us', 'ru-ru', 'es-es']}
            locale="en-us"
            domain="com"
          />);

          expect(wrapper2).toMatchSnapshot();
        });
      });

      describe('and `defaultLocales` prop is default one', () => {
        test('should match snapshot', () => {
          const wrapper2 = shallow(<LangSwitcher
            localesIntl={localesIntl}
            locale="en-us"
            domain="com"
          />);

          expect(wrapper2).toMatchSnapshot();
        });
      });
    });

    describe('when `localesIntl` prop is NOT valid string with spec structure', () => {
      describe('and `defaultLocales` prop is Object', () => {
        test('should match snapshot', () => {
          const wrapper2 = shallow(<LangSwitcher
            localesIntl=""
            defaultLocales={{
              'en-us': 'com',
              'ru-ru': 'ru',
              'es-es': 'es',
              'zh-cn': 'net',
              'ro-ro': 'ro',
              'ko-kr': 'kr',
              'pl-pl': 'pl',
              'fr-fr': 'fr',
            }}
            locale="en-us"
            domain="com"
          />);

          expect(wrapper2).toMatchSnapshot();
        });
      });

      describe('and `defaultLocales` prop is Array', () => {
        test('should match snapshot', () => {
          const wrapper2 = shallow(<LangSwitcher
            localesIntl=""
            defaultLocales={['en-us', 'ru-ru', 'es-es']}
            locale="en-us"
            domain="com"
          />);

          expect(wrapper2).toMatchSnapshot();
        });
      });

      describe('and `defaultLocales` prop is default one', () => {
        test('should match snapshot', () => {
          const wrapper2 = shallow(<LangSwitcher
            localesIntl=""
            locale="en-us"
            domain="com"
          />);

          expect(wrapper2).toMatchSnapshot();
        });
      });
    });
  });

  describe('redirects', () => {
    const testChangeLocale = (domain, locale, newLocale) => {
      const wrapper2 = shallow(<LangSwitcher
        locale={locale}
        localesIntl=""
        defaultLocales={{
          'en-us': 'com',
          'ru-ru': 'ru',
          'es-es': 'es',
          'zh-cn': 'net',
          'ro-ro': 'ro',
          'ko-kr': 'kr',
          'pl-pl': 'pl',
          'fr-fr': 'fr',
        }}
        domain={domain}
      />);

      const event2 = {
        target: {
          value: newLocale,
        },
      };

      wrapper2.instance().changeLocale(event2);
    };

    test('should redirect to .RU when current DOMAIN is .ES', () => {
      getLocationOrigin.mockReturnValue('https://scorum.es');
      testChangeLocale('es', 'es-es', 'ru-ru');
      expect(changeLocation).toHaveBeenCalledWith('https://scorum.ru');
    });

    test('should redirect to .NET when current DOMAIN is .COM', () => {
      getLocationOrigin.mockReturnValue('https://scorum.com');
      testChangeLocale('com', 'en-us', 'zh-cn');
      expect(changeLocation).toHaveBeenCalledWith('https://scorum.net');
    });

    test('should redirect to .PL when current DOMAIN is .KR', () => {
      getLocationOrigin.mockReturnValue('https://scorum.kr');
      testChangeLocale('kr', 'ko-kr', 'pl-pl');
      expect(changeLocation).toHaveBeenCalledWith('https://scorum.pl');
    });

    test('should redirect to .FR when current DOMAIN is .RU', () => {
      getLocationOrigin.mockReturnValue('https://scorum.ru');
      testChangeLocale('ru', 'ru-ru', 'fr-fr');
      expect(changeLocation).toHaveBeenCalledWith('https://scorum.fr');
    });
  });

  describe('when defaultLocales is Object', () => {
    const event = {
      target: {
        value: 'ru-ru',
      },
    };

    test('should match snapshot', () => {
      const wrapper2 = shallow(<LangSwitcher
        locale="en-us"
        localesIntl=""
        defaultLocales={{ 'en-us': 'com', 'ru-ru': 'net', 'es-es': 'es' }}
        domain="com"
      />);
      wrapper2.instance().changeLocale(event);
      expect(wrapper2).toMatchSnapshot();
    });

    describe('when value is not existed locale', () => {
      test('should match snapshot', () => {
        const wrapper2 = shallow(<LangSwitcher
          locale="en-us"
          localesIntl=""
          defaultLocales={{ 'en-us': 'com', 'ru-ru': 'net', 'es-es': 'es' }}
          domain="com"
        />);
        wrapper2.instance().changeLocale({
          target: {
            value: 'loh-nah',
          },
        });
        expect(wrapper2).toMatchSnapshot();
      });
    });

    describe('when `isWithinHeader` prop is TRUE', () => {
      describe('and `isWhite` is FALSE (default one)', () => {
        test('should match snapshot', () => {
          const wrapper3 = shallow(<LangSwitcher
            locale="en-us"
            localesIntl=""
            defaultLocales={{ 'en-us': 'com', 'ru-ru': 'net', 'es-es': 'es' }}
            isWithinHeader
            domain="com"
          />);
          expect(wrapper3).toMatchSnapshot();
        });
      });

      describe('and `isWhite` is TRUE', () => {
        test('should match snapshot', () => {
          const wrapper3 = shallow(<LangSwitcher
            locale="en-us"
            localesIntl=""
            defaultLocales={{ 'en-us': 'com', 'ru-ru': 'net', 'es-es': 'es' }}
            isWithinHeader
            isWhite
            domain="com"
          />);
          expect(wrapper3).toMatchSnapshot();
        });
      });
    });
  });
});

describe('`parseLocalesIntlKey` function', () => {
  describe('when intl key\'s structure is VALID', () => {
    test('should be equal to expected result', () => {
      const localesIntlKey = `en-us|English 1|en|com|Test desc
ru-ru|Русский|ru|me
zh-cn|中文|zh|net
es-es|Español|es|es`;

      const expectedResult = {
        'en-us': {
          title: 'English 1',
          shortCode: 'en',
          domain: 'com',
        },
        'ru-ru': {
          title: 'Русский',
          shortCode: 'ru',
          domain: 'me',
        },
        'zh-cn': {
          title: '中文',
          shortCode: 'zh',
          domain: 'net',
        },
        'es-es': {
          title: 'Español',
          shortCode: 'es',
          domain: 'es',
        },
      };

      const result = parseLocalesIntlKey(localesIntlKey);
      expect(result).toEqual(expectedResult);
    });

    describe('when `errorHandler` param (func) is set', () => {
      test('should NOT trigger `errorHandler` func', () => {
        const localesIntlKey = `en-us|English 1|en|com|Test desc
ru-ru|Русский|ru|me
zh-cn|中文|zh|net
es-es|Español|es|es`;
        const errorHandlerMock = jest.fn();

        // Fix. Ramda's `is(Function)` detects that `jest.fn()`) is NOT a 'Function' type
        // That's why wrapper func should be used
        const errorHandlerMockWrapper = (...args) => errorHandlerMock(...args);

        parseLocalesIntlKey(localesIntlKey, errorHandlerMockWrapper);
        expect(errorHandlerMock).not.toHaveBeenCalled();
      });
    });
  });

  describe('when intl key\'s structure is NOT VALID completely', () => {
    test('should be equal to NULL', () => {
      const localesIntlKey = '';

      const result = parseLocalesIntlKey(localesIntlKey);
      expect(result).toBeNull();
    });

    describe('when `errorHandler` param (func) is set', () => {
      test('should trigger `errorHandler` func', () => {
        const localesIntlKey = '';
        const errorHandlerMock = jest.fn();

        // Fix. Ramda's `is(Function)` detects that `jest.fn()`) is NOT a 'Function' type
        // That's why wrapper func should be used
        const errorHandlerMockWrapper = (...args) => errorHandlerMock(...args);

        parseLocalesIntlKey(localesIntlKey, errorHandlerMockWrapper);
        expect(errorHandlerMock).toHaveBeenCalled();
      });
    });
  });

  describe('when intl key\'s structure is NOT VALID partly', () => {
    test('should be equal to expected result', () => {
      const localesIntlKey = `en-us|English|en|com|Test desc
ru-ru|Русский|ru1|me
zh-cn|中文|zh|net1
es-es1|Español|es|es|
ba-ba|Balkanski|ba|ba||
en-ph|Filipino||en|ph
dfsdfsdf
de-de|Deutsch|de|de`;

      const expectedResult = {
        'en-us': {
          title: 'English',
          shortCode: 'en',
          domain: 'com',
        },
        'de-de': {
          title: 'Deutsch',
          shortCode: 'de',
          domain: 'de',
        },
      };

      const result = parseLocalesIntlKey(localesIntlKey);
      expect(result).toEqual(expectedResult);
    });
  });
});
