import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { FormattedMessage } from 'react-intl';
import QRCode from 'qrcode.react';

import Button from 'reactstrap/lib/Button';

import { IconEth } from '../../icons/icon-eth/icon-eth';
import { IconScr } from '../../icons/icon-scr/icon-scr';
import { IconButtonLoader } from '../../icons/icon-button-loader/icon-button-loader';
import { IconDotPreloader } from '../../icons/icon-dot-preloader/icon-dot-preloader';
import { InputWithIcon } from '../../form/input-with-icon/input-with-icon';
import { InputWithClipboard } from '../../form/input-with-clipboard/input-with-clipboard';

import GA from '../../../helpers/ga';
import { checkValidAmountValue } from '../../../helpers/validate';
import { CHAR_CODES } from '../../../constants';

import './send-amount.scss';

const copyClickGA = () => GA.event({ category: 'BuySCR', action: '4Copy eth address' });

export class SendAmount extends Component {
  static propTypes = {
    rate: PropTypes.string,
    ethValue: PropTypes.string,
    scrValue: PropTypes.string,
    maxAmount: PropTypes.string,
    isLoading: PropTypes.bool,
    ethAddress: PropTypes.string,
    ethErrorMsg: PropTypes.string,
    scrErrorMsg: PropTypes.string,
    onEthChange: PropTypes.func.isRequired,
    onScrChange: PropTypes.func.isRequired,
    onDoneClick: PropTypes.func.isRequired,
    ethFraction: PropTypes.number.isRequired,
    scrFraction: PropTypes.number.isRequired,
    isEthLoading: PropTypes.bool,
    isScrLoading: PropTypes.bool,
    isMaxAmountLoading: PropTypes.bool,
  };

  static defaultProps = {
    rate: null,
    ethValue: '',
    scrValue: '',
    isLoading: false,
    maxAmount: null,
    ethAddress: null,
    ethErrorMsg: '',
    scrErrorMsg: '',
    isEthLoading: false,
    isScrLoading: false,
    isMaxAmountLoading: false,
  };

  onKeyPress = (evt) => {
    const { which, keyCode } = evt;
    const charCode = which || keyCode;
    if ((charCode > CHAR_CODES.CONTROL &&
      ((charCode < CHAR_CODES.ZERO && charCode !== CHAR_CODES.DOT)
      || charCode > CHAR_CODES.NINE))
    ) {
      evt.preventDefault();
    }
  }

  ethChangeHandler = (e) => {
    const { value } = e.target;

    if (value.length && !checkValidAmountValue(value, this.props.ethFraction)) {
      return;
    }

    this.props.onEthChange(value);
  }

  scrChangeHandler = (e) => {
    const { value } = e.target;

    if (value.length && !checkValidAmountValue(value, this.props.scrFraction)) {
      return;
    }

    this.props.onScrChange(value);
  }

  onDoneClick = () => {
    this.props.onDoneClick();
    GA.event({ category: 'BuySCR', action: '5Done' });
  };

  render() {
    const {
      ethValue, scrValue, rate, maxAmount, ethErrorMsg, scrErrorMsg, ethAddress, isLoading, isEthLoading,
      isScrLoading, isMaxAmountLoading,
    } = this.props;

    return (
      <div className="send-amount-step">
        <FormattedMessage id="universal.balance.refill.send-step.title">
          {text => <span className="h3 d-block text-center mb-4">{text}</span>}
        </FormattedMessage>

        <div className="qr mx-auto mb-4 border border-shade-s30 rounded position-relative">
          {isLoading ? <IconButtonLoader withAbsolutePosition /> : <QRCode className="m-1_5" value={ethAddress} />}
        </div>

        <InputWithClipboard value={ethAddress} isLoading={isLoading} onCopyClick={copyClickGA} />

        <div className="send-amount-step-divider my-3" />

        <div className="d-flex mb-4">
          <InputWithIcon
            icon={<IconEth />}
            value={ethValue}
            errorMsg={ethErrorMsg}
            onChange={this.ethChangeHandler}
            isLoading={isEthLoading}
            onKeyPress={this.onKeyPress}
          />
          <span className="h5 mx-1 mx-sm-1_5 pt-2 modal-content-text-hint">=</span>
          <InputWithIcon
            icon={<IconScr />}
            value={scrValue}
            errorMsg={scrErrorMsg}
            onChange={this.scrChangeHandler}
            isLoading={isScrLoading}
            onKeyPress={this.onKeyPress}
          />
        </div>

        <div className="d-flex justify-content-between">
          <span className="modal-content-text-hint">
            <FormattedMessage id="universal.balance.refill.calc-step.rate" />
          </span>
          {!rate && (isEthLoading || isScrLoading) ? <IconDotPreloader className="h5p dot-preloader-small" />
            : (
              <span className="h5p ml-1 text-nowrap">
                1 SCR = {rate} ETH
              </span>
          )}
        </div>

        <div className="send-amount-step-divider my-2" />

        <div className="d-flex justify-content-between">
          <span className="modal-content-text-hint">
            <FormattedMessage id="universal.balance.refill.calc-step.max.title" />
          </span>
          {isMaxAmountLoading ? <IconDotPreloader className="h5p dot-preloader-small" />
          : (
            <span className="h5p ml-1 text-nowrap">
              <FormattedMessage id="universal.balance.refill.calc-step.max.value" values={{ value: maxAmount }} />
            </span>
          )}
        </div>

        <Button outline block className="send-amount-step__done mt-4" onClick={this.onDoneClick}>
          <FormattedMessage id="universal.done" />
        </Button>
      </div>
    );
  }
}
